#!/usr/bin/python

"""
   NAME
   
       analize.py - Webalizer runner for RedG customers statistics.

   SYNOPSIS

       Please call this function from logrotate or via crontab.

   DESCRIPTION
   
       This program developed to make it possible to produce separated
       web statistics for each customer of RedG. Since all information
       is stored in 1 log file this program will separate log file on
       difference pieces for each virtual host defined in configuration.

       Configuration of virtual hosts which should produce web statistics
       is kept in Zope to make easier maintance of the virtual hosts.

   FILES
   
       webalizer.conf - standard file which gets used as template for
       all customer configurations of webalizer.

   AUTHOR
   
       Alex V. Koval (alex@koval.kharkov.ua).

   VERSION
   
       1.0

   CHANGELOG
  
       04.May.2003 publishing to Zope.org
       11.Dec.2002 first version (1.0) released
       09.Dec.2002 development started
       
"""


############## Configuration section ####################
CONF_HTTP_USER = 'backupagent'
CONF_HTTP_PASS = 'xxxxxxxxxx'
CONF_URL  = 'http://zope.xxxx.com/webalizer_conf'
OUTPUT_DIR='/var/www/virtual/stats'
WEBALIZER_CONF_TEMPLATE = '/etc/RedG-webalizer/webalizer.conf'
ZOPE_LOGFILE="/home/zope/current/var/Z2.log"
############## End of configuration section #############

import os
import string
import sys
import re

def readconfig(url,user,passwd):
    
    """ Reads the configuration using external program 'curl'
    and returns array of parsed lines

    config line format is:
    NAME VIRTUAL_DOMAIN

    this function returns an array of arrays of splitted by space entries
    """
    
    conf = os.popen('curl -s -u %s:%s %s' % (user,passwd,url))
    lineno=1
    config = []
    for line in conf.readlines():
        line=string.strip(line)
        if len(line)==0:
            continue
        conf_args=string.split(line)
        if len(conf_args)!=2:
            raise ValueError, "ERROR: format error for config file near line %d:\n%s" % (lineno,line)
            sys.exit()
        config.append(conf_args)
        lineno=lineno+1

    conf.close()
    return config

def createDirs(output_directory, config):
    """ All webalizer output info will be stored in output_directory/NAME.
    Before running webalizer we should make it sure that all NAME directories
    are existing, and create them if some does not exists
    """
    
    for configline in config:
        try:
            os.stat("%s/%s" % (output_directory,configline[0]))
        except OSError:
            os.mkdir("%s/%s" % (output_directory,configline[0]))

def createWebalizerConfigs(output_dir,templatefile,config):
    """ This function will create customer webalizer.conf files
    according to the template file given as argument
    """
    for configline in config:
        name=configline[0]
        ifile = open(templatefile,'r')
        ofile = open( "%s/%s/webalizer.conf" % (output_dir, name), "w+")
        for line in ifile.readlines():
            line = string.strip(line)
            if len(line) == 0:          # throw away empty lines
                continue
            elif line[0]=='#':          # throw away comments
                continue
            elif line[:7]=='LogFile':   # changing LogFile location
                ofile.write("LogFile         %s/%s/access_log\n" % (output_dir,name))
            elif line[:9]=='OutputDir': # changing OutputDir location
                ofile.write("OutputDir       %s/%s\n" % (output_dir,name))
            elif line[:11]=='ReportTitle':
                ofile.write("ReportTitle     %s\n" % configline[0])
            else:
                ofile.write("%s\n" % line)
        
        ifile.close()
        ofile.close()

def separateAccessLogEntries(access_log_path,output_dir,config):
    ifile = open(access_log_path, "r")
    # make the dictionary for NAMES
    ofiles = {} # key: NAME value: opened file descriptor
    regexps = [] # [compiled regexp, filedescr]
    for configline in config:
        if not ofiles.has_key(configline[0]):
            ofiles[configline[0]] = open("%s/%s/access_log" % (output_dir, configline[0]), "w+")
        regexps.append([re.compile(".*?%s.*" % configline[1]), ofiles[configline[0]]])
    ftpregexp1 = re.compile('.*?==>.*')
    ftpregexp2 = re.compile('.*?<==.*')

    # parsing the file, saving found lines to different ofiles
    for line in ifile.readlines():
        line = string.strip(line)
        if ftpregexp1.match(line) or ftpregexp2.match(line):
            continue # we do not need FTP records for WebAlizer
        for check in regexps:
            if check[0].match(line):
                check[1].write("%s\n" % line)
        
        
    # closing all opened ofiles
    for ofile in ofiles.keys():
        ofiles[ofile].close()
    
    ifile.close()

def runWebalizer(output_dir,config):
    for configline in config:
        os.popen('/usr/bin/webalizer -q -Q -c %s/%s/webalizer.conf' % (output_dir, configline[0]))

if __name__ == '__main__':
    
    # reading configuration from Zope 
    config = readconfig(CONF_URL, CONF_HTTP_USER, CONF_HTTP_PASS)

    # creating directories if some does not existing (if configuration cnaged,
    # for example new virtual host added, etc. please note that program will
    # not delete any directories, even if those are no more exists in
    # Zope configuration of RedG webalizer
    createDirs(OUTPUT_DIR,config)

    # all configs are created from template file 'webalizer.conf' which is
    # located in the same directory where the program is
    createWebalizerConfigs(OUTPUT_DIR, WEBALIZER_CONF_TEMPLATE, config)

    # here is the main part of the program: it reads line by line the
    # configuration file and stores parsed entries per customer log files.
    # all log files which were existing before get truncated to zero.
    separateAccessLogEntries(ZOPE_LOGFILE,OUTPUT_DIR,config)

    # running webalizer on new generated log files and webalizer.conf files
    # generated by this program
    runWebalizer(OUTPUT_DIR,config)

# the end
